/*  File: blxmain.c
 *  Author: Erik Sonnhammer
 *  Copyright (c) J Thierry-Mieg and R Durbin, 1999
 * -------------------------------------------------------------------
 * Acedb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * or see the on-line version at http://www.gnu.org/copyleft/gpl.txt
 * -------------------------------------------------------------------
 * This file is part of the ACEDB genome database package, written by
 * 	Richard Durbin (Sanger Centre, UK) rd@sanger.ac.uk, and
 *	Jean Thierry-Mieg (CRBM du CNRS, France) mieg@kaa.crbm.cnrs-mop.fr
 *
 * Description: BLXMAIN - Standalone calling program for blxview.
 * Exported functions: 
 * HISTORY:
 * Last edited: Nov 30 15:26 1999 (esr)
 * * Aug 26 16:57 1999 (fw): added this header
 * Created: Thu Aug 26 16:56:45 1999 (fw)
 * CVS info:   $Id: blxmain.c,v 1.37 2000/04/12 12:49:16 srk Exp $
 *-------------------------------------------------------------------
 */

#include "regular.h"
#include "graph.h"
#include "gex.h"
#include "blxview.h"

void main(int argc, char **argv)
{
    FILE        *seqfile, *FSfile;
    char         
	seqfilename[1000], FSfilename[1000], 
	*qseq=0, 
        *dummyseq=0, /* Needed for blxparser to handle both dotter and blixem */
	line[MAXLINE+1],
	ch, *cp, *cq, *c;
    MSP         *MSPlist=0;
    static char  
	opts[32]=" MBr Z   ",	/* 0 L|N|P|T|X
				   1 M
				   2 B|b
				   3 R|r (blxparser.c)
				   4    (available)
				   5 Z (blxparser.c)
				   6 I 
				   7 G (blxparser.c)
				   8    sorting mode
				   */
	qname[FULLNAMESIZE+1],
        dummyseqname[FULLNAMESIZE+1];
    extern char *blixemVersion;
    int          dispstart=0, install = 1, seqInSFS=0;
    Array arr = arrayCreate(5000, char);

#if !defined(MACINTOSH)
    int          optc;
    extern int   optind;
    extern char *optarg;
    char        *optstring="bF:hIilnpS:s:t";

    static char *cc_date = 
#if defined(__DATE__)
    __DATE__
#else
    ""
#endif
    ;

    char *usage;
    static char usageText[] = "\
\n\
 Blixem - display Blast matches as a multiple alignment.\n\
\n\
 Reference:  Sonnhammer ELL & Durbin R (1994). A workbench for Large Scale\n\
 Sequence Homology Analysis. Comput. Applic. Biosci. 10:301-307.\n\
\n\
  Usage: blixem [options] <sequencefile> <datafile> [X options] \n\
\n\
 Both <sequencefile> and <datafile> can be substituted by \"-\"\n\
 for reading from stdin (pipe).  If <sequencefile> is piped, the first\n\
 line should contain the sequence name and the second the sequence itself.\n\
\n\
 Options:\n\
 -s <mode>  Sorting mode at startup.\n\
               s = by Score\n\
               i = by Identity\n\
               n = by Name\n\
               p = by Position\n\
 -I         Inverted sorting order\n\
 -b         Don't start with Big Picture.\n\
 -S <#>     Start display at position #.\n\
 -F <file>  Read in query sequence and data from <file> (replaces sequencefile).\n\
 -h         Help and more options.\n\
\n\
 Some X options:\n\
 -acefont <font> Main font.\n\
 -font    <font> Menu font.\n\
\n\
 To make the datafile from blast output, run MSPcrunch with option -q.\n\n\
\n\
 by Erik.Sonnhammer@cgr.ki.se\n\
 Version";
    
    usage = messalloc(strlen(usageText) + strlen(blixemVersion) + strlen(cc_date) + 10);
    sprintf(usage, "%s %s, %s\n", usageText, blixemVersion, cc_date);

    while ((optc = getopt(argc, argv, optstring)) != EOF )
	switch (optc)
	{
	case 'b': opts[2] = 'b';         break;
	case 'F': 
	    seqInSFS = 1;        
	    strcpy(FSfilename, optarg);  break;
	case 'h': 
	    fprintf(stderr, "%s\
\n\
 o To pipe MSPcrunch output directly to Blixem, use \"-\"\n\
   as the second parameter ([datafile]).  Example:\n\
\n\
   MSPcrunch -q <my.blast_output> | blixem <my.seq> -\n\
\n\
\n\
 o The BLAST program (blastp, blastn, blastx, tblastn, tblastx)\n\
   is automatically detected from the Blast output header by MSPcrunch\n\
   and is passed on to Blixem in the seqbl format (-q).  If\n\
   your output lacks the header, set the program with these options:\n\
\n\
   -p    Use blastp output (blastx is default)\n\
   -n    Use blastn output\n\
   -t    Use tblastn output\n\
   -l    Use tblastx output\n\
\n", usage); 
	    exit(-1);
	                                 break;
	case 'I': opts[6] = 'I';         break;
	case 'i': install = 0;           break;
	case 'l': opts[0] = 'L';         break;
	case 'n': opts[0] = 'N';         break;
	case 'p': opts[0] = 'P';         break;
	case 'S': 
	    if (!(dispstart = atoi(optarg)))
		messcrash("Bad diplay start position: %s", optarg); 
	    opts[1] = ' ';
	                                 break;
	case 's': 
	    if (*optarg != 's' && *optarg != 'i' && *optarg != 'n' && *optarg != 'p') {
		fprintf(stderr,"Bad sorting mode: %s\n", optarg); 
		exit(1);
	    }
	    opts[8] = *optarg;
	                                 break;
	case 't': opts[0] = 'T';         break;
	default : messcrash("Illegal option");
	}

    if (argc-optind < 2 && !seqInSFS) {
	fprintf(stderr, "%s\n", usage); 
	exit(1);
    }

#else  /* Macintosh */
    printf("Seq file: "); scanf("%s", seqfilename);
    printf("FS file: "); scanf("%s", FSfilename);
#endif
 
    if (!seqInSFS) {

	strcpy(seqfilename, argv[optind]);
	strcpy(FSfilename, argv[optind+1]);

	if (!strcmp(seqfilename, "-")) {
	    seqfile = stdin;
	}
	else if(!(seqfile = fopen(seqfilename, "r"))) {
	    messcrash("Cannot open %s\n", seqfilename);
	}
	
	/* Read in query sequence */
	if (seqfile == stdin) {
	    /* Same file for query and FS data - sequence on first two lines */
	    Array arr = arrayCreate(5000, char);
	    int i=0;
	    
	    if (!fgets(line, MAXLINE, seqfile)) messcrash("Error reading seqFile");;
	    sscanf(line, "%s", qname);
	    
	    while ((ch = fgetc(seqfile)) != '\n') {
		/*if (isAlnRes(ch) || isAlnNonres(ch))*/
		array(arr, i++, char) = ch;
	    }
	    qseq = messalloc(arrayMax(arr)+1);
	    cq = qseq;
	    for (i = 0; i < arrayMax(arr);) *cq++ = arr(arr, i++, char);
	    arrayDestroy(arr);
	}
	else {
	    fseek(seqfile, 0, SEEK_END);
	    qseq = messalloc(ftell(seqfile)+1);
	    cq = qseq;
	    fseek(seqfile, 0, SEEK_SET);
	    while (!feof(seqfile))
		{
		    if (!fgets(line, MAXLINE, seqfile)) break;
		    if (c = (char *)strchr(line, '\n')) *c = 0;
		    
		    if (*line == '>') {
			strncpy(qname, line+1, 255); qname[255]=0;
			if (c = (char *)strchr(qname, ' ')) *c = 0;
			continue;
		    }
		    
		    for (cp = line; *cp; cp++) 
			/*if (isAlnRes(*cp) || isAlnNonres(*cp))*/ *cq++ = *cp;
		    *cq = 0;
		}
	    fclose(seqfile);
	}
    }

    if (!strcmp(FSfilename, "-")) {
	FSfile = stdin;
    }
    else if(!(FSfile = fopen(FSfilename, "r"))) {
	messcrash("Cannot open %s\n", FSfilename);
    }
	
    parseFS(&MSPlist, FSfile, opts, &qseq, qname, &dummyseq, dummyseqname);

    /* Add -install for private colormaps */
    if (install) argvAdd(&argc, &argv, "-install");
    graphInit(&argc, argv) ;
    gexInit(&argc, argv);

    graphLoop(blxview (qseq, qname, dispstart, 0, MSPlist, opts));
}
