/*  Last edited: Jul 31 15:07 2001 (rd) */

/* gx for "Gene eXchange" */

/** here is the interface as seen by the C code in the client program **/
/** in some cases the CORBA layer will be need to be a little different **/

/*********** basic types ************/

typedef enum { NO_STRAND = 0, DOWN, UP } Strand ;
typedef enum { NO_PHASE = 0, PHASE_0, PHASE_1, PHASE_2 } Phase ;
typedef enum { BASIC_FEATURE, HOMOL_FEATURE, EXON_FEATURE, TRANSCRIPT_FEATURE, VARIATION_FEATURE, INTRON_FEATURE, BOUNDARY_FEATURE } FeatureType ;
typedef enum { N_HOMOL, X_HOMOL, TX_HOMOL } HomolSubType ;
typedef enum { CLONE_END, 5_SPLICE, 3_SPLICE } BoundarySubType ;
/* as in BLAST*, i.e. target is DNA, Protein, DNA translated */
typedef enum { OK_STATUS = 0, NO_WRITE_ACCESS } GxStatus ;

/*********** opaque types for server handles ************/

typedef struct DatabaseStruct *GxDB ;
typedef struct SeqContext *GxContext ; /* should cache DB handle */
typedef unsigned int GxID ;

/*********** local types corresponding to CORBA structs ***********/

typedef struct {
  int s1, s2 ;			/* coords in input space */
  int r1, r2 ;			/* coords in output space */
} AlignBlock ;	/* this structure is identical to AlignBlock */

typedef struct {
  int x1, x2 ;		/* start, end */
  Strand strand ;
  Phase phase ;
  float score ;
  char *text ;
  GxID id ;
  union {
    struct {
      int y1, y2 ;		/* targetStart, targetEnd */
      Strand targetStrand ;
      Phase targetPhase ;	/* for TX_HOMOL */
      Array align ;		/* of AlignBlock */
			/* if align is null, then ungapped */
    } homol ;
    struct {
      int cdsStart, cdsEnd ;
      Phase cdsPhase ;	/* cdsPhase = NO_PHASE unless startNotFound */
      char endNotFound ;
      Array exons ;	/* of Feature */
    } transcript ;
    struct {
      GxID transcriptID ;
    } exon ;
  } z ;			/* does C require an identifier here, or
			   can I just refer to feat.homol.y1? */
} Feature ;

/* efficiency: 
   Core feature is 26 bytes assuming 1 byte per enum.
   The largest extensions under "z" only add 18 extra bytes, and many
   of the features (e.g. all homols) will need these anyway.  It doesn't
   seem worth separating the data structures and duplicating lots of calls,
   requiring switches etc.
*/

/****************/

typedef struct {
  char *category ;		/* GFF_feature = BioPerl primary_tag */
  char *source ;		/* GFF_source = ?? */
  BOOL isStrandSensitive ;
  BOOL isFrameSensitive ;	/* phase becomes important */
  BOOL isScore ;
  FeatureType type ;
  HomolSubType homolType ;	/* for Homols only */
  char *targetDataSource ;	/* for Homols only */
  BoundarySubType boundType ;	/* for Boundaries only */
  Array attributeList ;		/* of char*, allowable attributes for this fset */
} GxFeatureSetDataInfo ;

typedef struct {
  char *urlFormat ;	/* printf style format with %s for target name */
  char *fetchFormat ;	/* similar format, but to fetch sequence */
  char *removePrefix ;	/* prefix to remove from names before substituting in formats */
} GxTargetDataSource ;

/* Separate this out from FeatureSet, so can get information on where
   to find the sequence from other sources than the database.  I guess the
   fetch thing should also be accessed via URLget, and we should set up
   interfaces for that.
*/

typedef struct {		/* incomplete... */
  colour stuff ...
  float right_priority ;
  enum { BY_WIDTH, BY_OFFSET, BY_HISTOGRAM } drawStyle ;
  enum { OVERLAP, BUMP, CLUSTER } overlapStyle ;
  float width ;			/* column width */
  float min_mag, max_mag ;
  float min_score, max_score ;
  BOOL  showText ;
  BOOL  showUpStrand ;
} GxFeatureSetDrawInfo ;

/**********************************************/

GxDB gxOpenDatabase (char *locator, char *user, char *password) ;

GxContext gxContext (GxDB db, char* seqName, int start, int end) ;

  /* end = 0 for end_of_sequence, so (start, end) = (1,0) for whole sequence */

char* gxDNA (GxContext context) ; /* returns IUPAC plus '-' for blank */

Array gxAvailableFeatureSets (GxContext context) ;

  /* Array of char* of feature names */

GxFeatureSetDataInfo gxFeatureSetDataInfo (GxDB db, char *fsetName) ;

GxFeatureSetDrawInfo gxFeatureSetDrawInfo (GxDB db, char *fsetName) ;

GxTargetDataSource *gxTargetDataSource (GxDB db, char* dataSource) ;

Array gxGetFeatures (GxContext context, GxFeatureSet *fset) ;

  /* Array of Feature structs */

/**********************************************/
/***************** write back *****************/

/* There is no way to write a gene as such, nor a naked exon.
   If you store a transcript not overlapping any genes it creates a new gene.  
   If this new gene overlaps one or more history genes then it is marked as 
   having split from the most recent that it overlaps.  If you store one that 
   overlaps two or more live genes then it merges them.  All within an fset.

   In the interface, a "curate" action on a transcript does the following:
     - if identical to a curated transcript then Kill or AddAttribute
     - else options are StoreTranscript or ReplaceTranscript for each that 
       it overlaps.

   If you kill a transcript (including by replacement) and thereby make one 
   or more new genes (by removing a bridge), then what?  Try to look to 
   history to reconstruct the names of the generated objects? (because often
   edits restore to an earlier solution.)
*/

/* In the following, attributes is additional textual information in 
   attribute/value per line format (as for .ace files).  If you want to
   communicate gene_name hints/info back and forth with the server you
   can use this.  Also attributes to be assigned to the gene.  The client
   will just neutrally put these up and allow them to be edited, deleted
   and added to, and passed back to the server.
*/

GxStatus gxNewFeature (GxDB db, char *fsetName, GxID *dbidPtr,
		       Feature *feature, char *attributes) ;

GxStatus gxReplaceFeature (GxDB db, char *fsetName, GxID dbid,
			   Feature *feature, char *attributes) ;

	/* if feature is null, just replace attributes and vice versa */

GxStatus gxKillFeature (GxDB db, GxID dbid, char *attributes) ;

	/* may want to change attributes because "kill" will only archive,
	   depending on server behaviour */

/* need also */

char* gxGetAttributes (GxDB db, GxID dbid) ;

/******************* end of file ***********************/
