/*  File: peptide.h
 *  Author: Richard Durbin (rd@sanger.cam.ac.uk)
 *  Copyright (C) J Thierry-Mieg and R Durbin, 1994
 * -------------------------------------------------------------------
 * Acedb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * or see the on-line version at http://www.gnu.org/copyleft/gpl.txt
 * -------------------------------------------------------------------
 * This file is part of the ACEDB genome database package, written by
 * 	Richard Durbin (MRC LMB, UK) rd@mrc-lmb.cam.ac.uk, and
 *	Jean Thierry-Mieg (CRBM du CNRS, France) mieg@kaa.cnrs-mop.fr
 *
 * Description:
 * Exported functions:
 * HISTORY:
 * Last edited: Mar 15 13:29 2001 (edgrif)
 * Created: Wed May 11 01:49:18 1994 (rd)
 * CVS info:   $Id: peptide.h,v 1.19 2001/03/15 13:55:46 edgrif Exp $
 *-------------------------------------------------------------------
 */
#ifndef ACEDB_PEPTIDE_H
#define ACEDB_PEPTIDE_H

extern char pepDecodeChar[] ;	/* [0,21] -> A,C,D... */
extern signed char pepEncodeChar[] ;	/* A,C,D... -> [0,21] : -ve bad */
extern char *pepName[] ; /* A,C,D... -> Alanine, Cysteine... */
extern char *pepShortName[] ;	/* A,C,D... -> Ala, Cys... */
extern int  molecularWeight[] ;  /* A,C,D... -> 89, 121... */
extern float NH2pK[]  ; /* A,C,D... -> 9.87, 10.78... */
extern float COOHpK[] ; /* A,C,D... -> 2.35, 1.71... */
void pepDecodeArray (Array pep) ;
void pepEncodeArray (Array pep) ;

				/* main access routines */
Array peptideGet (KEY key) ;
Array peptideTranslate(KEY key, BOOL CDS_only) ;
BOOL peptideStore (KEY key, Array pep) ; /* fails if model is missing */

                                /* ace dumping routines */
BOOL peptideDump (ACEOUT dump_out, KEY k) ;/* DumpFuncType */
ParseFuncReturn peptideParse (ACEIN parse_in, KEY key, char **errtext) ; /* ParseFuncType */

				/* FastA dumping routines */
BOOL pepDumpFastA (Array a, int from, int to, char *text, ACEOUT dump_out);
BOOL pepDumpFastAKey (ACEOUT dump_out, KEY key);
int  pepDumpFastAKeySet (ACEOUT dump_out, KEYSET kSet);

ACEOUT pepFileOpen (STORE_HANDLE handle);

int hashArray(Array a);


/* THIS SHOULD NOT BE HERE...SIGH...BUT CAN'T PUT IT IN PEPDISP.H BECAUSE    */
/* OF HEADER CLASHES...                                                      */
/* Pep display create data, controls how pep display will be created, passed */
/* in as a void * to pepDisplay() via displayApp()                           */
typedef struct _PepDisplayData
{
  BOOL CDS_only ;					    /* TRUE: translate only CDS portion of */
							    /* objects DNA.*/
} PepDisplayData ;



#endif   /* !ACEDB_PEPTIDE_H */
/************** end of file **************/
 
 
